@@ -3,6 +3,9 @@
 set -e
 set -x
 
+uname -a
+python -c "import sys; print(sys.version)"
+
 if [[ "$(uname -s)" == 'Darwin' ]]; then
     brew update || brew update
     brew outdated pyenv || brew upgrade pyenv
@@ -13,22 +16,22 @@ if [[ "$(uname -s)" == 'Darwin' ]]; then
     fi
 
     case "${PYVER}" in
-        py26)
-            pyenv install 2.6.9
-            pyenv virtualenv 2.6.9 psutil
-            ;;
+        # py26)
+        #     pyenv install 2.6.9
+        #     pyenv virtualenv 2.6.9 psutil
+        #     ;;
         py27)
             pyenv install 2.7.10
             pyenv virtualenv 2.7.10 psutil
             ;;
-        py32)
-            pyenv install 3.2.6
-            pyenv virtualenv 3.2.6 psutil
-            ;;
-        py33)
-            pyenv install 3.3.6
-            pyenv virtualenv 3.3.6 psutil
-            ;;
+        # py32)
+        #     pyenv install 3.2.6
+        #     pyenv virtualenv 3.2.6 psutil
+        #     ;;
+        # py33)
+        #     pyenv install 3.3.6
+        #     pyenv virtualenv 3.3.6 psutil
+        #     ;;
         py34)
             pyenv install 3.4.3
             pyenv virtualenv 3.4.3 psutil
